/*
Copy contents of complete first row to all other rows, useful for fixing inconsistent album metadata
for example
*/
function copyRow(key)
{
    var table = document.getElementById(key+"table");
    var rows  = table.getElementsByTagName("tr");
    var row1  = rows[1]; //Row 0 is header row
    var list1 = row1.getElementsByTagName("input");

    if(rows.length>2)
    {
        for(i =2; i <= rows.length; i++)
        {
            var list = rows[i].getElementsByTagName("input");
            for (var j = 0; j < list1.length; j++)
            {
                if(list[j].type == "checkbox")
                {
                    list[j].checked=list1[j].checked;
                }
                else
                {
                    list[j].value=list1[j].value;
                }
            }
        }
    }
}

/*
Copy contents of this field in row 1 to all other rows, useful for fixing inconsistent album metadata
for example
*/
function copyColRow(key, col)
{
    var table = document.getElementById(key+"table");
    var rows  = table.getElementsByTagName("tr");
    var row1  = rows[1]; //Row 0 is header row
    var list1 = row1.getElementsByTagName("input");

    if(rows.length>2)
    {
        for(i =2; i <= rows.length; i++)
        {
            var list = rows[i].getElementsByTagName("input");
            if(list[col].type == "checkbox")
            {
                list[col].checked=list1[col].checked;
            }
            else
            {
                list[col].value=list1[col].value;
            }
        }
    }
}

/**
Resize Column Code
*/
function each(arr, fn)
{
    let i = arr.length
    while(--i > -1)
    {
        fn(arr[i])
    }
} 

function px(val)
{
    return [val, 'px'].join("")
}

var resizeElement, startSize, startX;

function beginResize(e)
{
    killResize()
    let th = e.target.parentElement
    resizeElement = th
    startSize = th.clientWidth
    startX = e.pageX
} 

function killResize()
{
    resizeElement = null
    startSize = null
    startX = null
} 

each(document.querySelectorAll('th'),
    elem =>
    {
        let trigger = document.createElement('span')
        trigger.className = 'resizeTrigger'
        trigger.addEventListener('mousedown', beginResize)
        elem.appendChild(trigger)
    }
)

document.addEventListener('mousemove', e =>
{
    if(resizeElement)
    {
        let diff = e.pageX - startX
        resizeElement.style.width = px(startSize + diff)
    }
})

document.addEventListener('mouseup', killResize)

/** Drag/Drop Images from another webpage
*/
var dropzone    = document.getElementById('res'),
send_btn        = document.getElementById('send'),
res             = document.getElementById('res'),
resCaption      = document.getElementById('resCaption'),
img;

dropzone.ondragover
    = function ondragover(e)
    {
        e.preventDefault();
        dropzone.classList.add('dragover');
    };

dropzone.ondrop
    = function ondrop(e)
    {
        e.preventDefault();
        dropzone.classList.remove('dragover');

        // try to get images from this dropevent
        img = retrieveImageData(e.dataTransfer);
        if (!img)
        {
            return;
        }
        res.src=img.element.src;
    };

dropzone.ondragexit
    = function(ondragexit)
    {
        dropzone.classList.remove('dragover');
    };

/**
User selects local file on computer to upload
*/
function uploadImageFile(filelist)
{
    img =
    {
        type: 'file',
        element: new Image(),
        file: filelist[0]
    };
    img.element.src = URL.createObjectURL(img.file);
    img.element.onload
        = function() {
        res.src=img.element.src;
        resCaption.innerHTML=img.element.width + " x " + img.element.height + " (" + img.file.type.split('/').pop().toUpperCase() + ")";
    }
}

/**
User specifies a url to upload image from
*/
function uploadUrl()
{
    img =
    {
        type: 'element',
        element: new Image()
    };
    img.element.src = document.getElementById("paste_artwork_url").value;
    res.src = img.element.src;
    resCaption.innerHTML=res.width + " x " + res.height;
}

/**
Get Data from latest user drop
*/
function retrieveImageData(dataTransfer)
{
    // first try to get Files
    var img = getFile(dataTransfer);
    if (img!=null)
    {
        return img;
    }

    // if none, try to get HTMLImage or SVGImage
    var elems = getHTMLMarkup(dataTransfer);
    if (elems && elems.length)
    {
        return elems[0];
    }
    // we could also try to getData('text/plain') hoping for an url
    // but this might not be such a good idea...
    console.warn('unable to retrieve any image in dropped data');
}

/** User has drag/dropped a local file, if they try and drop more than one we just take the first one
*/
function getFile(dataTransfer)
{
    if (dataTransfer.files && dataTransfer.files.length)
    {
        // only Image Files
        if (dataTransfer.files[0].type.indexOf('image/') === 0)
        {
            imgObj =
            {
                type: 'file',
                element: new Image(),
                file: dataTransfer.files[0]
            };
            imgObj.element.src = URL.createObjectURL(imgObj.file);
            imgObj.element.onload = function() {
                resCaption.innerHTML=imgObj.element.width + " x " + imgObj.element.height + " (" + imgObj.file.type.split('/').pop().toUpperCase() + ")";
              }
            return imgObj;
        }
    }
}


/**
Dragged an image from another webpage

NOTE DO NOT TRY TO REFORMAT SEEMS TO BREAK IT
*/
function getHTMLMarkup(dT)
{
    var markup = dT.getData('text/html');
    if (markup)
    {
        var doc = new DOMParser().parseFromString(markup, 'text/html');
        var imgs = doc && doc.querySelectorAll('img,image') ||  [];
        imgs.forEach(toImageObject);
        return Array.prototype.map.call(imgs, toImageObject);
    }

  function toImageObject(element) {
    var img;
    if (element instanceof SVGImageElement) {
      img = new Image();

      img.onload = function() {
          resCaption.innerHTML=img.width + " x " + img.height;
      }
      img.src = element.getAttributeNS('http://www.w3.org/1999/xlink', 'href') ||
        element.getAttribute('href');
    } else {
      img = document.adoptNode(element);
       img.onload = function() {
                resCaption.innerHTML=img.width + " x " + img.height;
            }
    }
    return {
      type: 'element',
      element: img
    };
  }
}

// Once we got everything, time to retrieve our objects
function sendData()
{
    if(img.type == 'file')
    {
        var reader  = new FileReader();
        reader.addEventListener("load",
            function ()
            {
                sendAjax(reader.result);
            }, false);

        reader.readAsDataURL(img.file);
    }
    else if(img.type == 'element')
    {
        sendAjax(img.element.src);
    }
};

function sendAjax(data)
{
    console.log('Data', data);

    var xhr = new XMLHttpRequest();

    xhr.onreadystatechange = function() {
        if (this.readyState == 4 && this.status == 200)
        {
            var check       = document.getElementById("ARTWORK_TABLE").querySelectorAll("input[type='checkbox']")
            var images      = document.getElementById("ARTWORK_TABLE").getElementsByTagName("img");
            var imglabel    = document.getElementById("ARTWORK_TABLE").getElementsByClassName("imglabel");
            var hidden      = document.getElementById("ARTWORK_TABLE").querySelectorAll("input[type='hidden']")

            if(xhr.responseText.length > 0)
            {
                var response = JSON.parse(xhr.responseText);
                for (var i = 0; i < check.length; i++)
                {
                    if(check[i].checked==true)
                    {
                        images[i].src   = "images/" + response.path + ".jpg";
                        images[i].title = response.dim;
                        imglabel[i].textContent = response.dim;
                        hidden[i].value = response.path;
                    }
                }
            }
        }
    };

    xhr.open('POST', '/editsongs.update_artwork', true);
    xhr.setRequestHeader("Content-type", "text/plain");
    xhr.send(data);
}


/**
Let user change row in table using up/down keys
*/
function upTo(el, tagName) {
  tagName = tagName.toLowerCase();

  while (el && el.parentNode) {
    el = el.parentNode;
    if (el.tagName && el.tagName.toLowerCase() == tagName) {
      return el;
    }
  }

  // Many DOM methods return null if they don't
  // find the element they are searching for
  // It would be OK to omit the following and just
  // return undefined
  return null;
}

document.onkeydown =
    function  updownintable (e)
    {
        switch (e.key)
        {
            case 'ArrowUp':
                var el = document.activeElement;
                var rowNo   = el.id.substring(0,el.id.indexOf("_"));
                var fieldId = el.id.substring(el.id.indexOf("_"));
                if(rowNo>1)
                {
                    rowNo--;
                    var newId   = rowNo + fieldId;
                    var newEl   = document.getElementById(newId);
                    if(newEl!=null)
                    {
                        newEl.focus();
                    }
                }
                break;
            case 'ArrowDown':
                var el = document.activeElement;
                var rowNo   = el.id.substring(0,el.id.indexOf("_"));
                var fieldId = el.id.substring(el.id.indexOf("_"));
                rowNo++
                var newId   = rowNo + fieldId;
                var newEl   = document.getElementById(newId);
                if(newEl!=null)
                {
                    newEl.focus();
                }
                break;
        }
    };


/**
* Invalid data verification
*/
function invalidData(event, msg)
{
    var alerts = document.getElementById("Alerts");
    var alert = document.createElement("div");
    var msg   = document.createTextNode(msg);
    alert.className = "alert alert-danger";
    alert.appendChild(msg);
    alerts.appendChild(alert);
}

/**
Clear all alerts
*/
function clearAllAlerts()
{
    var alerts = document.getElementById("Alerts");
    while (alerts.hasChildNodes()) {
      alerts.removeChild(alerts.lastChild);
    }
}